@@ -26,33 +26,12 @@
 #ifndef __PPP_MANAGER__
 #define __PPP_MANAGER__
 
+#include <net/if.h>
+
 //#define PRINTF(x) 	printf x
 #define PRINTF(x)
 
-
-u_long 	ppp_init_all();
-struct ppp *ppp_new(CFStringRef serviceID, CFStringRef subtypeRef);
-void ppp_updatesetup(struct ppp *ppp, CFDictionaryRef service);
-void ppp_updatestate(struct ppp *ppp, CFDictionaryRef service);
-void ppp_printlist();
-void ppp_postupdatesetup();
-int ppp_dispose(struct ppp *ppp);
-struct ppp *ppp_findbyname(u_char *name, u_short unit);
-struct ppp *ppp_findbyserviceID(CFStringRef serviceID);
-struct ppp *ppp_findbyref(u_long ref);
-struct ppp *ppp_findbysid(u_char *data, int len);
-void ppp_setorder(struct ppp *ppp, u_int16_t order);
-u_short ppp_findfreeunit(u_short subfam);
-u_int32_t ppp_makeref(struct ppp *ppp);
-u_int32_t ppp_makeifref(struct ppp *ppp);
-int ppp_logout();
-int ppp_login();
-int ppp_logswitch();
-int ppp_doconnect(struct ppp *ppp, CFDictionaryRef opts, u_int8_t dialondemand, void *client, int autoclose);
-int ppp_dodisconnect(struct ppp *ppp, int sig, void *client);
-int ppp_dosuspend(struct ppp *ppp);
-int ppp_doresume(struct ppp *ppp);
-int ppp_clientgone(void *client);
+//#define DEBUG 1
 
 /* ppp service client, used for arbitration */
 struct ppp_client {
@@ -61,6 +40,19 @@ struct ppp_client {
     int 	autoclose;
 };
 
+enum {
+    FLAG_SETUP = 0x1,		/* needs to process service setup */
+    FLAG_FREE = 0x2,		/* needs to dispose of the ppp structure */
+    FLAG_CONNECT = 0x4,		/* needs to connect service */
+    FLAG_CONFIGCHANGEDNOW = 0x8,	/* setup has changed, dialondemand needs to rearm with no delay */
+    FLAG_CONFIGCHANGEDLATER = 0x10,	/* setup has changed, dialondemand needs to rearm with delay if applicable */
+    FLAG_DIALONDEMAND = 0x20,	/* is the connection currently in dial-on-demand mode */
+    FLAG_ALERTERRORS = 0x40,	/* error alerts are enabled */
+    FLAG_ALERTPASSWORDS = 0x80,	/* passwords alerts are enabled */
+   // FLAG_STARTING = 0x100	/* pppd is started, and hasn't yet updated the phase */
+	FLAG_FIRSTDIAL = 0x200 /* is it the first autodial attempt after major event */
+};
+
 /* this struct contains all the information to control a ppp interface */
 struct ppp {
 
@@ -74,40 +66,62 @@ struct ppp {
     u_int16_t 	subtype;		/* ppp subtype of link */
     u_int16_t 	unit;			/* ref number in the interfaces managed by this Controller */
     
-    // name/ifunit make the ifnet interface
-    u_char      name[IFNAMSIZ];		/* real ifname */
-    u_int16_t 	ifunit;			/* real ifunit number */
-    
     // status information frequently used
     u_int32_t 	phase;			/* where the link is at */    
-    u_int32_t 	oldphase;		/* where the link was at last lime */    
-    u_int32_t 	conntime; 		/* time when connected	*/
-    u_int32_t 	disconntime; 		/* time when disconnection is planned, 
-                                            as advertised by the server, or as required by the session timer */
+    u_char      ifname[IFNAMSIZ];	/* real ifname */
+    CFStringRef device;				/* transport device (en0, en1,...) */
+    int			ndrv_socket;		/* ndrv socket to maintain transport device up */
     u_int32_t 	laststatus;		/* last fail status */
     u_int32_t 	lastdevstatus;		/* last device specific fail status */
-    u_int32_t 	alertenable;		/* alert level flags */
     CFBundleRef	bundle;			/* PPP device bundle */
 
-    int	iFD[2];				/* pipe for pppd params */
+    int		controlfd[2];		/* pipe for pppd control */
+    int		statusfd[2];		/* pipe for pppd status */
 
-    u_int8_t	dialondemand;		/* is pppd curently running in dialondemand mode ? */
-    u_int8_t	started_link;		/* pppd has just been started */
-    u_int8_t	kill_link;		/* pppd needs to be killed when it appears (sig number) */
-    u_int8_t	kill_sent;		/* kill signal has been sent, wait for idle */
-    u_int8_t	dosetup;		/* needs to process service setup */
-    u_int8_t	needconnect;		/* needs to process service setup */
-    u_int8_t	needdispose;		/* needs to dispose of the ppp structure */
-    u_int8_t    setupchanged;           /* setup has changed, dialondemand needs to rearm */
+	uid_t   uid;				/* uid of the user who started the connection */
+	gid_t   gid;				/* gid of the user who started the connection */
+	mach_port_t   bootstrap;	/* bootstrap of the user who started the connection */
+	
+    u_int32_t	flags;			/* action flags */
     pid_t     	pid;                    /* pid of associated pppd */
-    CFDictionaryRef needconnectopts; 	/* connect options to use */ 
     CFDictionaryRef connectopts; 	/* connect options in use */ 
+    CFDictionaryRef newconnectopts; 	/* new connect options to use */ 
+    uid_t		newconnectuid; 	/* new connect uid */ 
+    gid_t		newconnectgid; 	/* new connect gid */ 
+    mach_port_t		newconnectbootstrap; 	/* new connect bootstrap */ 
 
     // list of clients for this service. used to arbitrate connection/disconnection
     TAILQ_HEAD(, ppp_client) 	client_head;
 };
 
 
+u_long 	ppp_init_all();
+void ppp_stop_all(CFRunLoopSourceRef stopRls);
+
+void ppp_updatestatus(struct ppp *ppp, int status, int devstatus);
+void ppp_updatephase(struct ppp *ppp, int phase);
+
+struct ppp *ppp_findbyserviceID(CFStringRef serviceID);
+struct ppp *ppp_findbyref(u_long ref);
+struct ppp *ppp_findbysid(u_char *data, int len);
+struct ppp *ppp_findbypid(pid_t pid);
+u_int32_t ppp_makeref(struct ppp *ppp);
+
+int ppp_connect(struct ppp *ppp, CFDictionaryRef options, u_int8_t dialondemand, void *client, int autoclose, uid_t uid, gid_t gid, mach_port_t bootstrap);
+int ppp_disconnect(struct ppp *ppp, void *client, int signal);
+int ppp_suspend(struct ppp *ppp);
+int ppp_resume(struct ppp *ppp);
+int ppp_getstatus (struct ppp *ppp, void **reply, u_int16_t *replylen);
+int ppp_copyextendedstatus (struct ppp *ppp, void **reply, u_int16_t *replylen);
+int ppp_copystatistics(struct ppp *ppp, void **reply, u_int16_t *replylen);
+int ppp_getconnectdata(struct ppp *ppp, void **reply, u_int16_t *replylen, int all);
+int ppp_getconnectsystemdata(struct ppp *ppp, void **reply, u_int16_t *replylen);
+
+u_int32_t ppp_translate_error(u_int16_t subtype, u_int32_t native_ppp_error, u_int32_t native_dev_error);
+
+int ppp_clientgone(void *client);
+
+
 extern CFURLRef 	gBundleURLRef;
 extern CFBundleRef 	gBundleRef;
 extern CFStringRef 	gCancelRef;
