@@ -91,8 +91,8 @@ static int  ppp_if_demux(ifnet_t ifp, mbuf_t m, char *frame_header,
 static int  ppp_if_add_proto(ifnet_t ifp, protocol_family_t protocol_family,
 			const struct ifnet_demux_desc *demux_list, u_int32_t demux_count);
 static int  ppp_if_del_proto(ifnet_t ifp, protocol_family_t protocol_family);
-static int  ppp_if_frameout(ifnet_t ifp, mbuf_t *m0,
-                     struct sockaddr *ndest, char *edst, char *ppp_type);
+static errno_t  ppp_if_frameout(ifnet_t ifp, mbuf_t *m0,
+                     const struct sockaddr *ndest, const char *edst, const char *ppp_type);
 
 static int 	ppp_if_detach(ifnet_t ifp);
 static struct ppp_if *ppp_if_findunit(u_short unit);
@@ -904,7 +904,7 @@ errno_t ppp_if_output(ifnet_t ifp, mbuf_t m)
 			ifnet_stat_increment(ifp, &statsinc);		
             return ENOBUFS;
         }
-        *(u_int16_t*)mbuf_data(m) = 0xFF03;
+        *(u_int16_t*)mbuf_data(m) = htons(0xFF03);
 		(*wan->bpf_output)(ifp, m);
         mbuf_adj(m, 2);
     }
@@ -1007,8 +1007,8 @@ a network packet needs to be send through the interface.
 add the ppp header to the packet (as a network interface, we only worry
 about adding our protocol number)
 ----------------------------------------------------------------------------- */
-int ppp_if_frameout(ifnet_t ifp, mbuf_t *m0,
-                     struct sockaddr *ndest, char *edst, char *ppp_type)
+errno_t ppp_if_frameout(ifnet_t ifp, mbuf_t *m0,
+                     const struct sockaddr *ndest, const char *edst, const char *ppp_type)
 {
 	struct		ifnet_stat_increment_param statsinc;
 
@@ -1082,7 +1082,7 @@ int ppp_if_detachlink(struct ppp_link *link)
 int ppp_if_send(ifnet_t ifp, mbuf_t m)
 {
     struct ppp_if 	*wan = ifnet_softc(ifp);
-    u_int16_t		proto = *(u_int16_t*)mbuf_data(m);	// always the 2 first bytes
+    u_int16_t		proto = ntohs(*(u_int16_t*)mbuf_data(m));	// always the 2 first bytes
 	struct			ifnet_stat_increment_param statsinc;
 	
 	lck_mtx_assert(ppp_domain_mutex, LCK_MTX_ASSERT_OWNED);
@@ -1109,17 +1109,17 @@ int ppp_if_send(ifnet_t ifp, mbuf_t m)
                     mp = mbuf_next(mp);
                     if (!mp)
                         break;
-                    ip = mbuf_data(m);
+                    ip = mbuf_data(mp);
                 }
                 // this code assumes the IP/TCP header is in one non-shared mbuf 
                 if (ip->ip_p == IPPROTO_TCP) {
                     vjtype = sl_compress_tcp(mp, ip, wan->vjcomp, !(wan->sc_flags & SC_NO_TCP_CCID));
                     switch (vjtype) {
                         case TYPE_UNCOMPRESSED_TCP:
-                            *(u_int16_t*)mbuf_data(m) = PPP_VJC_UNCOMP; // update protocol
+                            *(u_int16_t*)mbuf_data(m) = htons(PPP_VJC_UNCOMP); // update protocol
                             break;
                         case TYPE_COMPRESSED_TCP:
-                            *(u_int16_t*)mbuf_data(m) = PPP_VJC_COMP; // header has moved, update protocol
+                            *(u_int16_t*)mbuf_data(m) = htons(PPP_VJC_COMP); // header has moved, update protocol
                         break;
                     }
                     // adjust packet len
@@ -1146,7 +1146,7 @@ int ppp_if_send(ifnet_t ifp, mbuf_t m)
 				ifnet_stat_increment(ifp, &statsinc);		
                 return ENOBUFS;
             }
-            *(u_int16_t*)mbuf_data(m) = PPP_COMP; // update protocol
+            *(u_int16_t*)mbuf_data(m) = htons(PPP_COMP); // update protocol
         } 
     } 
 
