@@ -23,23 +23,46 @@
 /*
  * lcp.c - PPP Link Control Protocol.
  *
- * Copyright (c) 1989 Carnegie Mellon University.
- * All rights reserved.
+ * Copyright (c) 1984-2000 Carnegie Mellon University. All rights reserved.
  *
- * Redistribution and use in source and binary forms are permitted
- * provided that the above copyright notice and this paragraph are
- * duplicated in all such forms and that any documentation,
- * advertising materials, and other materials related to such
- * distribution and use acknowledge that the software was developed
- * by Carnegie Mellon University.  The name of the
- * University may not be used to endorse or promote products derived
- * from this software without specific prior written permission.
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any legal
+ *    details, please contact
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#define RCSID	"$Id: lcp.c,v 1.8.22.1 2003/11/20 18:54:18 callie Exp $"
+#define RCSID	"$Id: lcp.c,v 1.13 2005/03/11 05:48:31 lindak Exp $"
 
 /*
  * TODO:
@@ -52,10 +75,12 @@
 #include "pppd.h"
 #include "fsm.h"
 #include "lcp.h"
-#include "chap.h"
+#include "chap-new.h"
 #include "magic.h"
 
+#ifndef __APPLE__
 static const char rcsid[] = RCSID;
+#endif
 
 /*
  * When the link comes up we want to be able to wait for a short while,
@@ -376,6 +401,7 @@ lcp_init(unit)
     ao->neg_chap = 1;
     ao->chap_mdtype = MDTYPE_ALL;
     ao->neg_upap = 1;
+    ao->neg_eap = 1;
     ao->neg_magicnumber = 1;
     ao->neg_pcompression = 1;
     ao->neg_accompression = 1;
@@ -415,6 +441,17 @@ lcp_close(unit, reason)
 {
     fsm *f = &lcp_fsm[unit];
 
+#ifdef __APPLE__
+    /* 
+     * if close is requested very early, 
+     * we need to update the state machine first 
+     */
+	if (f->flags & DELAYED_UP) {
+		untimeout(lcp_delayed_up, f);
+		lcp_delayed_up(f);
+    }
+#endif
+
     if (phase != PHASE_DEAD)
 	new_phase(PHASE_TERMINATE);
     if (f->state == STOPPED && f->flags & (OPT_PASSIVE|OPT_SILENT)) {
@@ -427,8 +464,14 @@ lcp_close(unit, reason)
 	f->state = CLOSED;
 	lcp_finished(f);
 
-    } else
-	fsm_close(&lcp_fsm[unit], reason);
+    } else {
+	fsm_close(f, reason);
+#ifdef __APPLE__
+	/* should be done in fsm, but it has side effect on ncps */ 
+	if (f->state == INITIAL)
+		lcp_finished(f);
+#endif
+	}
 }
 
 
@@ -447,9 +490,10 @@ lcp_lowerup(unit)
      * but accept A/C and protocol compressed packets
      * if we are going to ask for A/C and protocol compression.
      */
-    ppp_send_config(unit, PPP_MRU, 0xffffffff, 0, 0);
-    ppp_recv_config(unit, PPP_MRU, (lax_recv? 0: 0xffffffff),
-		    wo->neg_pcompression, wo->neg_accompression);
+    if (ppp_send_config(unit, PPP_MRU, 0xffffffff, 0, 0) < 0
+	|| ppp_recv_config(unit, PPP_MRU, (lax_recv? 0: 0xffffffff),
+			   wo->neg_pcompression, wo->neg_accompression) < 0)
+	    return;
     peer_mru[unit] = PPP_MRU;
 
 #ifdef __APPLE__
@@ -515,7 +559,6 @@ lcp_input(unit, p, len)
 	f->flags &= ~DELAYED_UP;
 	fsm_lowerup(f);
     }
-
     fsm_input(f, p, len);
 }
 
@@ -690,16 +733,15 @@ lcp_cilen(f)
 #define LENCILQR(neg)	((neg) ? CILEN_LQR: 0)
 #define LENCICBCP(neg)	((neg) ? CILEN_CBCP: 0)
     /*
-     * NB: we only ask for one of CHAP and UPAP, even if we will
-     * accept either.
+     * NB: we only ask for one of CHAP, UPAP, or EAP, even if we will
+     * accept more than one.  We prefer EAP first, then CHAP, then
+     * PAP.
      */
     return (LENCISHORT(go->neg_mru && go->mru != DEFMRU) +
 	    LENCILONG(go->neg_asyncmap && go->asyncmap != 0xFFFFFFFF) +
-	    LENCICHAP(go->neg_chap) +
-	    LENCISHORT(!go->neg_chap && go->neg_upap) +
-#ifdef EAP
-	    LENCISHORT(!go->neg_chap && !go->neg_upap && go->neg_eap) +
-#endif
+	    LENCISHORT(go->neg_eap) +
+	    LENCICHAP(!go->neg_eap && go->neg_chap) +
+	    LENCISHORT(!go->neg_eap && !go->neg_chap && go->neg_upap) +
 	    LENCILQR(go->neg_lqr) +
 	    LENCICBCP(go->neg_cbcp) +
 	    LENCILONG(go->neg_magicnumber) +
@@ -773,11 +815,10 @@ lcp_addci(f, ucp, lenp)
     ADDCISHORT(CI_MRU, go->neg_mru && go->mru != DEFMRU, go->mru);
     ADDCILONG(CI_ASYNCMAP, go->neg_asyncmap && go->asyncmap != 0xFFFFFFFF,
 	      go->asyncmap);
-    ADDCICHAP(CI_AUTHTYPE, go->neg_chap, go->chap_mdtype);
-    ADDCISHORT(CI_AUTHTYPE, !go->neg_chap && go->neg_upap, PPP_PAP);
-#ifdef EAP
-    ADDCISHORT(CI_AUTHTYPE, !go->neg_chap && !go->neg_upap && go->neg_eap, PPP_EAP);
-#endif
+    ADDCISHORT(CI_AUTHTYPE, go->neg_eap, PPP_EAP);
+    ADDCICHAP(CI_AUTHTYPE, !go->neg_eap && go->neg_chap, go->chap_mdtype);
+    ADDCISHORT(CI_AUTHTYPE, !go->neg_eap && !go->neg_chap && go->neg_upap,
+	       PPP_PAP);
     ADDCILQR(CI_QUALITY, go->neg_lqr, go->lqr_period);
     ADDCICHAR(CI_CALLBACK, go->neg_cbcp, CBCP_OPT);
     ADDCILONG(CI_MAGICNUMBER, go->neg_magicnumber, go->magicnumber);
@@ -923,11 +964,10 @@ lcp_ackci(f, p, len)
     ACKCISHORT(CI_MRU, go->neg_mru && go->mru != DEFMRU, go->mru);
     ACKCILONG(CI_ASYNCMAP, go->neg_asyncmap && go->asyncmap != 0xFFFFFFFF,
 	      go->asyncmap);
-    ACKCICHAP(CI_AUTHTYPE, go->neg_chap, go->chap_mdtype);
-    ACKCISHORT(CI_AUTHTYPE, !go->neg_chap && go->neg_upap, PPP_PAP);
-#ifdef EAP
-    ACKCISHORT(CI_AUTHTYPE, !go->neg_chap && !go->neg_upap && go->neg_eap, PPP_EAP);
-#endif
+    ACKCISHORT(CI_AUTHTYPE, go->neg_eap, PPP_EAP);
+    ACKCICHAP(CI_AUTHTYPE, !go->neg_eap && go->neg_chap, go->chap_mdtype);
+    ACKCISHORT(CI_AUTHTYPE, !go->neg_eap && !go->neg_chap && go->neg_upap,
+	       PPP_PAP);
     ACKCILQR(CI_QUALITY, go->neg_lqr, go->lqr_period);
     ACKCICHAR(CI_CALLBACK, go->neg_cbcp, CBCP_OPT);
     ACKCILONG(CI_MAGICNUMBER, go->neg_magicnumber, go->magicnumber);
@@ -1062,6 +1102,10 @@ lcp_nakci(f, p, len)
 	try.neg = 0; \
     }
 
+    /*
+     * NOTE!  There must be no assignments to individual fields of *go in
+     * the code below.  Any such assignment is a BUG!
+     */
     /*
      * We don't care if they want to send us smaller packets than
      * we want.  Therefore, accept any MRU less than what we asked for,
@@ -1090,49 +1134,40 @@ lcp_nakci(f, p, len)
      * they are proposing a different protocol, or a different
      * hash algorithm for CHAP.
      */
-    if ((go->neg_chap 
-#ifdef EAP
-        || go->neg_eap
-#endif
-        || go->neg_upap)
+    if ((go->neg_chap || go->neg_upap || go->neg_eap)
 	&& len >= CILEN_SHORT
 	&& p[0] == CI_AUTHTYPE && p[1] >= CILEN_SHORT && p[1] <= len) {
 	cilen = p[1];
 	len -= cilen;
 	no.neg_chap = go->neg_chap;
 	no.neg_upap = go->neg_upap;
-#ifdef EAP
 	no.neg_eap = go->neg_eap;
-#endif
 	INCPTR(2, p);
 	GETSHORT(cishort, p);
 	if (cishort == PPP_PAP && cilen == CILEN_SHORT) {
+	    /* If we were asking for EAP, then we need to stop that. */
+	    if (go->neg_eap)
+		try.neg_eap = 0;
+
+	    /* If we were asking for CHAP, then we need to stop that. */
+	    else if (go->neg_chap)
+		try.neg_chap = 0;
 	    /*
-	     * If we were asking for CHAP, they obviously don't want to do it.
-	     * If we weren't asking for CHAP, then we were asking for PAP,
-	     * in which case this Nak is bad.
+	     * If we weren't asking for CHAP or EAP, then we were asking for
+	     * PAP, in which case this Nak is bad.
 	     */
-	    if (!go->neg_chap
-#ifdef EAP
-                && !go->neg_eap
-#endif
-                )
-                goto bad;
-	    try.neg_chap = 0;
-#ifdef EAP
-	    try.neg_eap = 0;
-#endif
+	    else
+		goto bad;
 
-#ifdef EAP
-	} else if (cishort == PPP_EAP && cilen == CILEN_SHORT) {
-	    if (!go->neg_chap && !go->neg_eap)
-                goto bad;
-	    try.neg_chap = 0;
-	    try.neg_upap = 0;
-#endif
 	} else if (cishort == PPP_CHAP && cilen == CILEN_CHAP) {
 	    GETCHAR(cichar, p);
-	    if (go->neg_chap) {
+	    /* Stop asking for EAP, if we were. */
+	    if (go->neg_eap) {
+		try.neg_eap = 0;
+		/* Try to set up to use their suggestion, if possible */
+		if (CHAP_CANDIGEST(go->chap_mdtype, cichar))
+		    try.chap_mdtype = CHAP_MDTYPE_D(cichar);
+	    } else if (go->neg_chap) {
 		/*
 		 * We were asking for our preferred algorithm, they must
 		 * want something different.
@@ -1140,7 +1175,7 @@ lcp_nakci(f, p, len)
 		if (cichar != CHAP_DIGEST(go->chap_mdtype)) {
 		    if (CHAP_CANDIGEST(go->chap_mdtype, cichar)) {
 			/* Use their suggestion if we support it ... */
-			go->chap_mdtype = CHAP_MDTYPE_D(cichar);
+			try.chap_mdtype = CHAP_MDTYPE_D(cichar);
 		    } else {
 			/* ... otherwise, try our next-preferred algorithm. */
 			try.chap_mdtype &= ~(CHAP_MDTYPE(try.chap_mdtype));
@@ -1159,22 +1194,25 @@ lcp_nakci(f, p, len)
 		 * Stop asking for PAP if we were asking for it.
 		 */
 		try.neg_upap = 0;
-#ifdef EAP
-                try.neg_eap = 0;
-#endif
 	    }
 
 	} else {
+
+	    /*
+	     * If we were asking for EAP, and they're Conf-Naking EAP,
+	     * well, that's just strange.  Nobody should do that.
+	     */
+	    if (cishort == PPP_EAP && cilen == CILEN_SHORT && go->neg_eap)
+		dbglog("Unexpected Conf-Nak for EAP");
+
 	    /*
 	     * We don't recognize what they're suggesting.
 	     * Stop asking for what we were asking for.
 	     */
-	    if (go->neg_chap)
-		try.neg_chap = 0;
-#ifdef EAP
 	    if (go->neg_eap)
-                try.neg_eap = 0;
-#endif
+		try.neg_eap = 0;
+	    else if (go->neg_chap)
+		try.neg_chap = 0;
 	    else
 		try.neg_upap = 0;
 	    p += cilen - CILEN_SHORT;
@@ -1255,7 +1293,7 @@ lcp_nakci(f, p, len)
      * An option we don't recognize represents the peer asking to
      * negotiate some option we don't support, so ignore it.
      */
-    while (len > CILEN_VOID) {
+    while (len >= CILEN_VOID) {
 	GETCHAR(citype, p);
 	GETCHAR(cilen, p);
 	if (cilen < CILEN_VOID || (len -= cilen) < 0)
@@ -1279,11 +1317,8 @@ lcp_nakci(f, p, len)
 		goto bad;
 	    break;
 	case CI_AUTHTYPE:
-	    if (go->neg_chap || no.neg_chap || go->neg_upap || no.neg_upap
-#ifdef EAP
-                || go->neg_eap || no.neg_eap
-#endif            
-                )
+	    if (go->neg_chap || no.neg_chap || go->neg_upap || no.neg_upap ||
+		go->neg_eap || no.neg_eap)
 		goto bad;
 	    break;
 	case CI_MAGICNUMBER:
@@ -1409,7 +1444,7 @@ lcp_rejci(f, p, len)
 	if ((cishort != PPP_CHAP) || (cichar != (CHAP_DIGEST(val)))) \
 	    goto bad; \
 	try.neg = 0; \
-	try.neg_upap = 0; \
+	try.neg_eap = try.neg_upap = 0; \
     }
 #define REJCILONG(opt, neg, val) \
     if (go->neg && \
@@ -1472,13 +1507,13 @@ lcp_rejci(f, p, len)
 
     REJCISHORT(CI_MRU, neg_mru, go->mru);
     REJCILONG(CI_ASYNCMAP, neg_asyncmap, go->asyncmap);
-    REJCICHAP(CI_AUTHTYPE, neg_chap, go->chap_mdtype);
-    if (!go->neg_chap) {
-	REJCISHORT(CI_AUTHTYPE, neg_upap, PPP_PAP);
-    }
-#ifdef EAP
     REJCISHORT(CI_AUTHTYPE, neg_eap, PPP_EAP);
-#endif
+    if (!go->neg_eap) {
+	REJCICHAP(CI_AUTHTYPE, neg_chap, go->chap_mdtype);
+	if (!go->neg_chap) {
+	    REJCISHORT(CI_AUTHTYPE, neg_upap, PPP_PAP);
+	}
+    }
     REJCILQR(CI_QUALITY, neg_lqr, go->lqr_period);
     REJCICBCP(CI_CALLBACK, neg_cbcp, CBCP_OPT);
     REJCILONG(CI_MAGICNUMBER, neg_magicnumber, go->magicnumber);
@@ -1614,34 +1649,30 @@ lcp_reqci(f, inp, lenp, reject_if_disagree)
 
 	case CI_AUTHTYPE:
 	    if (cilen < CILEN_SHORT ||
-#ifdef EAP
-                !(ao->neg_eap) &&
-#endif            
-		!(ao->neg_upap || ao->neg_chap)) {
+		!(ao->neg_upap || ao->neg_chap || ao->neg_eap)) {
 		/*
 		 * Reject the option if we're not willing to authenticate.
 		 */
+		dbglog("No auth is possible");
 		orc = CONFREJ;
 		break;
 	    }
 	    GETSHORT(cishort, p);
 
 	    /*
-	     * Authtype must be PAP or CHAP.
+	     * Authtype must be PAP, CHAP, or EAP.
 	     *
-	     * Note: if both ao->neg_upap and ao->neg_chap are set,
-	     * and the peer sends a Configure-Request with two
-	     * authenticate-protocol requests, one for CHAP and one
-	     * for UPAP, then we will reject the second request.
-	     * Whether we end up doing CHAP or UPAP depends then on
+	     * Note: if more than one of ao->neg_upap, ao->neg_chap, and
+	     * ao->neg_eap are set, and the peer sends a Configure-Request
+	     * with two or more authenticate-protocol requests, then we will
+	     * reject the second request.
+	     * Whether we end up doing CHAP, UPAP, or EAP depends then on
 	     * the ordering of the CIs in the peer's Configure-Request.
              */
 
 	    if (cishort == PPP_PAP) {
-		if (ho->neg_chap ||	/* we've already accepted CHAP */
-#ifdef EAP
-		    ho->neg_eap ||	/* we've already accepted EAP */
-#endif
+		/* we've already accepted CHAP or EAP */
+		if (ho->neg_chap || ho->neg_eap ||
 		    cilen != CILEN_SHORT) {
 		    LCPDEBUG(("lcp_reqci: rcvd AUTHTYPE PAP, rejecting..."));
 		    orc = CONFREJ;
@@ -1649,36 +1680,37 @@ lcp_reqci(f, inp, lenp, reject_if_disagree)
 		}
 		if (!ao->neg_upap) {	/* we don't want to do PAP */
 		    orc = CONFNAK;	/* NAK it and suggest CHAP or EAP */
-                    PUTCHAR(CI_AUTHTYPE, nakp);
+		    PUTCHAR(CI_AUTHTYPE, nakp);
 		    if (ao->neg_eap) {
-                        PUTCHAR(CILEN_SHORT, nakp);
-                        PUTSHORT(PPP_EAP, nakp);
-                    }
-                    else {
-                        PUTCHAR(CILEN_CHAP, nakp);
-                        PUTSHORT(PPP_CHAP, nakp);
-                        PUTCHAR(CHAP_DIGEST(ao->chap_mdtype), nakp);
-                    }
+			PUTCHAR(CILEN_SHORT, nakp);
+			PUTSHORT(PPP_EAP, nakp);
+		    } else {
+			PUTCHAR(CILEN_CHAP, nakp);
+			PUTSHORT(PPP_CHAP, nakp);
+			PUTCHAR(CHAP_DIGEST(ao->chap_mdtype), nakp);
+		    }
 		    break;
 		}
 		ho->neg_upap = 1;
 		break;
 	    }
 	    if (cishort == PPP_CHAP) {
-		if (ho->neg_upap ||	/* we've already accepted PAP */
-#ifdef EAP
-		    ho->neg_eap ||	/* we've already accepted EAP */
-#endif
+		/* we've already accepted PAP or EAP */
+		if (ho->neg_upap || ho->neg_eap ||
 		    cilen != CILEN_CHAP) {
 		    LCPDEBUG(("lcp_reqci: rcvd AUTHTYPE CHAP, rejecting..."));
 		    orc = CONFREJ;
 		    break;
 		}
 		if (!ao->neg_chap) {	/* we don't want to do CHAP */
-		    orc = CONFNAK;	/* NAK it and suggest PAP */
+		    orc = CONFNAK;	/* NAK it and suggest EAP or PAP */
 		    PUTCHAR(CI_AUTHTYPE, nakp);
 		    PUTCHAR(CILEN_SHORT, nakp);
-		    PUTSHORT(PPP_PAP, nakp);
+		    if (ao->neg_eap) {
+			PUTSHORT(PPP_EAP, nakp);
+		    } else {
+			PUTSHORT(PPP_PAP, nakp);
+		    }
 		    break;
 		}
 		GETCHAR(cichar, p);	/* get digest type */
@@ -1698,11 +1730,9 @@ lcp_reqci(f, inp, lenp, reject_if_disagree)
 		ho->neg_chap = 1;
 		break;
 	    }
-#ifdef EAP
 	    if (cishort == PPP_EAP) {
-		if (ho->neg_chap ||	/* we've already accepted CHAP */
-		    ho->neg_upap ||	/* we've already accepted PAP */
-                    cilen != CILEN_SHORT) {
+		/* we've already accepted CHAP or PAP */
+		if (ho->neg_chap || ho->neg_upap || cilen != CILEN_SHORT) {
 		    LCPDEBUG(("lcp_reqci: rcvd AUTHTYPE EAP, rejecting..."));
 		    orc = CONFREJ;
 		    break;
@@ -1711,39 +1741,38 @@ lcp_reqci(f, inp, lenp, reject_if_disagree)
 		    orc = CONFNAK;	/* NAK it and suggest CHAP or PAP */
 		    PUTCHAR(CI_AUTHTYPE, nakp);
 		    if (ao->neg_chap) {
-                        PUTCHAR(CILEN_CHAP, nakp);
-                        PUTSHORT(PPP_CHAP, nakp);
-                        PUTCHAR(CHAP_DIGEST(ao->chap_mdtype), nakp);
-                    }
-                    else {
-                        PUTCHAR(CILEN_SHORT, nakp);
-                        PUTSHORT(PPP_PAP, nakp);
-                    }
-		    /* XXX if we can do CHAP_MICROSOFT as well, we should
-		       probably put in another option saying so */
+			PUTCHAR(CILEN_CHAP, nakp);
+			PUTSHORT(PPP_CHAP, nakp);
+#ifdef __APPLE__
+			PUTCHAR(CHAP_DIGEST(ao->chap_mdtype), nakp);
+#else
+			PUTCHAR(ao->chap_mdtype, nakp);
+#endif
+		    } else {
+			PUTCHAR(CILEN_SHORT, nakp);
+			PUTSHORT(PPP_PAP, nakp);
+		    }
 		    break;
 		}
 		ho->neg_eap = 1;
 		break;
 	    }
-#endif            
 
 	    /*
 	     * We don't recognize the protocol they're asking for.
 	     * Nak it with something we're willing to do.
-	     * (At this point we know ao->neg_upap || ao->neg_chap.)
+	     * (At this point we know ao->neg_upap || ao->neg_chap ||
+	     * ao->neg_eap.)
 	     */
 	    orc = CONFNAK;
 	    PUTCHAR(CI_AUTHTYPE, nakp);
-	    if (ao->neg_chap) {
+	    if (ao->neg_eap) {
+		PUTCHAR(CILEN_SHORT, nakp);
+		PUTSHORT(PPP_EAP, nakp);
+	    } else if (ao->neg_chap) {
 		PUTCHAR(CILEN_CHAP, nakp);
 		PUTSHORT(PPP_CHAP, nakp);
 		PUTCHAR(CHAP_DIGEST(ao->chap_mdtype), nakp);
-#ifdef EAP
-	    } else if (ao->neg_eap) {
-		PUTCHAR(CILEN_SHORT, nakp);
-		PUTSHORT(PPP_EAP, nakp);
-#endif
 	    } else {
 		PUTCHAR(CILEN_SHORT, nakp);
 		PUTSHORT(PPP_PAP, nakp);
@@ -2094,7 +2123,7 @@ lcp_printpkt(p, plen, printer, arg)
 			printer(arg, "chap");
 			if (p < optend) {
 			    switch (*p) {
-			    case CHAP_DIGEST_MD5:
+			    case CHAP_MD5:
 				printer(arg, " MD5");
 				++p;
 				break;
@@ -2112,6 +2141,9 @@ lcp_printpkt(p, plen, printer, arg)
 			    }
 			}
 			break;
+		    case PPP_EAP:
+			printer(arg, "eap");
+			break;
 		    default:
 			printer(arg, "0x%x", cishort);
 		    }
@@ -2437,4 +2469,4 @@ lcp_received_timeremaining (f, id, inp, len)
     notify(lcp_timeremaining_notify, (int)&info);
 
 }
-#endif
\ No newline at end of file
+#endif
