@@ -116,6 +116,7 @@ int main (int argc, char *argv[])
 	exit(EXIT_NOT_ROOT);
     }
     
+
     params = (struct vpn_params*)malloc(sizeof (struct vpn_params));
     if (params == 0)
         exit(EXIT_FATAL_ERROR);
@@ -509,7 +510,7 @@ void vpnlog(int nSyslogPriority, char *format_str, ...)
 static void dump_params(struct vpn_params *params)
 {
     int	i;
-	char	*subtype = 0, *servertype;
+	char	*subtype = 0, *servertype = "Unknown";
     
 	switch (params->server_type) {
 		case SERVER_TYPE_IPSEC:
